/*
ID: espr1t
TASK: Closest Points
KEYWORDS: Solver
*/

#include <cstdio>
#include <cmath>
#include <vector>
#include <algorithm>
#include <ctime>
#include <cassert>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

#define DEBUG_INFO false

const int MAX = 100000;
const int DIM = 5;
const long long INF = 1000000000000000001LL;

int n, k, q;
int a[MAX][DIM], b[MAX][DIM];

inline long long sqr(long long a) {
    return a * a;
}

long long fdist(int idxB, int idxA) {
    long long ret = 0;
    for (int i = 0; i < k; i++)
        ret += sqr(a[idxA][i] - b[idxB][i]);
    return ret;
}

vector <long long> smarty() {
    // For each coordinate, look for the 100 closest by this coordinate points
    const int DIST = 100;
    vector <long long> res;

    vector < pair <int, int> > sorted[DIM];
    for (int dim = 0; dim < k; dim++) {
        for (int i = 0; i < n; i++)
            sorted[dim].push_back(make_pair(a[i][dim], i));
        sort(sorted[dim].begin(), sorted[dim].end());
    }
    
    for (int i = 0; i < q; i++) {
        long long minn = INF;
        for (int dim = 0; dim < k; dim++) {
            int left = 0, right = (int)sorted[dim].size() - 1;
            while (left <= right) {
                int mid = (left + right) / 2;
                if (sorted[dim][mid].first < b[i][dim])
                    left = mid + 1;
                else
                    right = mid - 1;
            }
            for (int c = max(0, right - DIST); c < min(n, right + DIST); c++) {
                long long cur = 0;
                for (int d = 0; d < k; d++)
                    cur += sqr(b[i][d] - a[sorted[dim][c].second][d]);
                if (minn > cur) {
                    minn = cur;
                    if (DEBUG_INFO) {
                        fprintf(stderr, "  -- found new closest point to %d: %d (dist = %.9lf)\n",
                            i, sorted[dim][c].second, sqrt((double)cur));
                    }
                }
            }
        }
        res.push_back(minn);
    }
    return res;
}

vector <long long> dummy() {
    vector <long long> res;
    for (int i = 0; i < q; i++) {
        long long minn = INF;
        for (int c = 0; c < n; c++) {
            long long cur = 0;
            for (int dim = 0; dim < k; dim++)
                cur += sqr(a[c][dim] - b[i][dim]);
            if (minn > cur) {
                minn = cur;
                if (DEBUG_INFO) {
                    fprintf(stderr, "  -- found new closest point to %d: %d (dist = %.9lf)\n",
                        i, c, sqrt((double)cur));
                }
            }
        }
        res.push_back(minn);
    }
    return res;
}


template <typename T> class KDTree {
    public:
        struct Node {
            int left, right;
            vector <T> coords;
            Node(const vector<T>& coords_) : coords(coords_) {}
        };
    
    private:
        int numDimensions_;
        vector <Node> tree_; int root_;
    
        int buildTree_(vector < vector<T> >& points, int L, int R, int level) {
            if (R < L)
                return -1;
            int dim = level % numDimensions_;
                
            vector <T> v(R - L + 1);
            for (int i = L; i <= R; i++)
                v[i - L] = points[i][dim];                
            nth_element(v.begin(), v.begin() + v.size() / 2, v.end());
            T median = v[v.size() / 2];

            int idx = L;
            for (int i = L; i <= R; i++) {
                if (points[i][dim] < median) {
                    swap(points[idx], points[i]);
                    idx++;
                }
            }
            for (int i = idx; i <= R; i++) {
                if (points[i][dim] == median) {
                    swap(points[idx], points[i]);
                    break;
                }
            }
            
            int ret = (int)tree_.size();
            tree_.push_back(Node(points[idx]));
            int leftChild = buildTree_(points, L, idx - 1, level + 1);
            int rightChild = buildTree_(points, idx + 1, R, level + 1);
            tree_[ret].left = leftChild, tree_[ret].right = rightChild;
            return ret;
        }
        
        void closestPoint_(int node, int level, const vector<T>& point, long long& best, vector<T>& res) {
            if (node == -1)
                return;
            
            long long dist = 0;
            for (int dim = 0; dim < numDimensions_; dim++)
                dist += sqr(point[dim] - tree_[node].coords[dim]);
            if (best > dist)
                best = dist, res = tree_[node].coords;

            int dim = level % numDimensions_;
            bool descendLeft = point[dim] < tree_[node].coords[dim];
            closestPoint_(descendLeft ? tree_[node].left : tree_[node].right, level + 1, point, best, res);
            if (best > sqr(tree_[node].coords[dim] - point[dim]))
                closestPoint_(descendLeft ? tree_[node].right : tree_[node].left, level + 1, point, best, res);
        }
    
    public:
        KDTree(int K, vector < vector<T> > points = vector < vector<T> >()) {
            numDimensions_ = K;
            root_ = buildTree_(points, 0, (int)points.size() - 1, 0);
        }
        
        vector <T> closestPoint(const vector<T>& point) {
            vector <T> res;
            long long best = INF;
            closestPoint_(root_, 0, point, best, res);
            return res;
        }
};


vector <long long> solve() {
    vector <long long> res;
    
    if (DEBUG_INFO) {
        fprintf(stderr, "Building tree...\n");
    }
    
    vector < vector <int> > points;
    for (int i = 0; i < n; i++)
        points.push_back(vector<int>(a[i], a[i] + k));
    KDTree<int> kdtree(k, points);

    for (int i = 0; i < q; i++) {
        if (DEBUG_INFO) {
            fprintf(stderr, "  >> answering query %d for point (", i);
            for (int dim = 0; dim < k; dim++)
                fprintf(stderr, "%d%s", b[i][dim], dim + 1 == k ? ")...\n" : ", ");
        }
        vector <int> point(b[i], b[i] + k);
        vector<int> closest = kdtree.closestPoint(point);
        if (DEBUG_INFO) {
            fprintf(stderr, "    -- found point (");
            for (int dim = 0; dim < (int)closest.size(); dim++)
                fprintf(stderr, "%d%s", closest[dim], dim + 1 == (int)closest.size() ? ")\n" : ", ");
        }
        long long dist = 0;
        for (int dim = 0; dim < (int)closest.size(); dim++) {
            dist += sqr(point[dim] - closest[dim]);
        }
        res.push_back(dist);
    }
    return res;
}

void runSolutions(FILE* out) {
    unsigned sTime;

    sTime = clock();
    vector <long long> resDummy = dummy();
    fprintf(stderr, "Dummy execution time: %.3lfs\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);
    
    sTime = clock();
    vector <long long> resSmarty = smarty();
    fprintf(stderr, "Smarty execution time: %.3lfs\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);

    sTime = clock();

    vector <long long> resSolve = solve();
    
    fprintf(stderr, "Solve execution time: %.3lfs\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);

    if (resDummy != resSmarty) {
        fprintf(stderr, "WARNING: dummy() returns different result than smarty()!\n");
    }
    if (resDummy != resSolve) {
        fprintf(stderr, "ERROR: dummy() returns different result than solve()!\n");
        for (int i = 0; i < (int)resDummy.size(); i++) {
            if (resDummy[i] != resSolve[i]) {
                fprintf(stderr, "  -- first diference at query %d: %.3lf vs %.3lf\n", i, sqrt((double)resDummy[i]), sqrt((double)resSolve[i]));
                break;
            }
        }
    }
    
    for (int i = 0; i < (int)resSolve.size(); i++)
        fprintf(out, "%.3lf\n", sqrt((double)resSolve[i]));
}

void readInput(FILE* in) {
    fscanf(in, "%d %d", &n, &k);
    for (int i = 0; i < n; i++)
        for (int c = 0; c < k; c++)
            fscanf(in, "%d", &a[i][c]);
    fscanf(in, "%d", &q);
    for (int i = 0; i < n; i++)
        for (int c = 0; c < k; c++)
            fscanf(in, "%d", &b[i][c]);
}

int main(void) {
    int numTests = 20;
    for (int test = 0; test <= numTests; test++) {
        unsigned sTime = clock();
        fprintf(stderr, "Solving test %d...\n", test);

        char inpFile[32], outFile[32];
        sprintf(inpFile, "ClosestPoints.%02d.in", test);
        sprintf(outFile, "ClosestPoints.%02d.sol", test);

        FILE* in = fopen(inpFile, "rt"); readInput(in); fclose(in);
        FILE* out = fopen(outFile, "wt"); runSolutions(out); fclose(out);

        fprintf(stderr, "    -- execution time: %.3lfs\n",
            (double)(clock() - sTime) / CLOCKS_PER_SEC);
    }
    fprintf(stderr, "Finished solving all tests.\n");
    return 0;
}
